from enum import Enum

class EUMember(Enum):
    """
    An Enum class used to map a country name (key) to it's country code (value).
    """
    AUSTRIA        = 'AT'
    BELGIUM        = 'BE'
    BULGARIA       = 'BG'
    CROATIA        = 'HR'
    CYPRUS         = 'CY'
    CZECH_REPUBLIC = 'CZ'
    DENMARK        = 'DK'
    ESTONIA        = 'EE'
    FINLAND        = 'FI'
    FRANCE         = 'FR'
    GERMANY        = 'DE'
    GREECE         = 'GR'
    HUNGARY        = 'HU'
    IRELAND        = 'IE'
    ITALY          = 'IT'
    LATVIA         = 'LV'
    LITHUANIA      = 'LT'
    LUXEMBOURG     = 'LU'
    MALTA          = 'MT'
    NETHERLANDS    = 'NL'
    POLAND         = 'PO'
    PORTUGAL       = 'PT'
    ROMANIA        = 'RO'
    SLOVAKIA       = 'SK'
    SLOVENIA       = 'SL'
    SPAIN          = 'ES'
    SWEDEN         = 'SE'
    UNITED_KINGDOM = 'GB'
